<?php

if ( ! class_exists('Disable_Floc_Easily_Admin') ) {
    class Disable_Floc_Easily_Admin
    {
        public $instance = 'disable-floc-easily-admin';

        // Permissions key which be added later
        public $headerKey = 'Permissions-Policy';

        // Value for Permissions Policy header
        public $headerValue = 'interest-cohort=()';

        public function __construct()
        {
            add_action( 'init', array( $this, 'dfe_initialize_floc_blocking_methods' ) );
            add_filter('dfe_blocking_methods_filter', array( $this, 'disable_floc_blocking_methods' ) );
            add_action('admin_menu', array( $this, 'add_settings_page' ) );
            add_action('admin_init', array( $this, 'register_settings_sections_and_fields' ) );
            add_filter( 'pre_update_option_dfe_general_settings', array( $this, 'dfe_flush_rewrite_rules' ), 10, 3 );
            add_action('admin_enqueue_scripts', array( $this, 'dfe_admin_enqueue_scripts' ));
        }

        public function dfe_initialize_floc_blocking_methods()
        {
            $options  = get_option( 'dfe_general_settings' );
            $selectedOption = $options['dfe_general_settings_select_method'];
	        $methods = apply_filters( 'dfe_blocking_methods_filter', array() );
	
	        if (
                is_array( $methods ) &&
                isset( $methods[ $selectedOption ] ) &&
                isset( $methods[ $selectedOption ]['callback'] )
            ) {
	            call_user_func( $methods[ $selectedOption ]['callback'] );
	        }
        }

        public function add_settings_page()
        {
            add_options_page( 
                __('Disable FLoC Settings', 'disable-floc-easily'), 
                __( 'Disable FLoC', 'disable-floc-easily' ), 
                'manage_options', 
                'disable-floc-easily', 
                array( $this, 'disable_floc_settings_markup' ) 
            );
        }

        public function register_settings_sections_and_fields()
        {
            register_setting(
                'dfe_general', 
                'dfe_general_settings', 
                array(
                    'default'   => 'filter',
                ) 
            );
            // Flush rules if Apache method is set
            if ( get_transient('dfe_flush_rules') == 'true' ) {
                flush_rewrite_rules();
                delete_transient('dfe_flush_rules');
             }

            add_settings_section( 
                'dfe_general_section', '', 
                null, 
                'dfe_general' 
            );

            add_settings_field( 
                'dfe_general_settings_method', 
                __('Select Method', 'disable-floc-easily'), 
                array( $this, 'dfe_general_settings_method_callback' ),
                'dfe_general', 
                'dfe_general_section' 
            );

            add_settings_field(  
                'dfe_delete_options',  
                __( 'Delete data when Uninstalled', 'disable-floc-easily' ),  
                array( $this, 'dfe_delete_options_callback'),  
                'dfe_general',  
                'dfe_general_section'  
            );
        }

        public function disable_floc_settings_markup()
        {
        ?>
            <div class="wg-wrapper">
                <header class="wg-p-header">
                    <div class="wg-p-header-inner">
                        <div class="wg-p-header-left">
                            <span class="dashicons dashicons-hidden"></span>
                            <h3><?php _e('Disable Floc Easily', 'disable-floc-easily'); ?></h3>
                        </div>
                        <img class="wg-p-header-right" src="<?php echo esc_url( DFE_PLUGIN_DIR_URI . 'assets/websiteguider_logo.png' ); ?>">
                    </div>
                </header>
                <div class="wg-p-body">
                    <div class="wg-p-inner">
                        <form method="post" action="options.php">
                            <?php
                                settings_fields( 'dfe_general' );
                                do_settings_sections( 'dfe_general' );
                                submit_button();
                            ?>
                        </form>
                    </div>
                    <sidebar class="wg-p-sidebar">
                        <div class="wg-affiliate-links">
                            <div class="wg-side-wrapper">
                            <h3><?php _e('Recommended Tools', 'disable-floc-easily'); ?></h3>
                                <ul>
                                    <li><a href="<?php echo esc_url( 'https://shareasale.com/r.cfm?b=1713706&u=2323235&m=97231&urllink=&afftrack=0'); ?>"><?php _e( 'SEMrush - Tools for Any SEO Challenge', 'disable-floc-easily' ); ?></a></li>
                                    <li><a href="<?php echo esc_url( 'https://mbsy.co/3jGCLB'); ?>"><?php _e( 'DreamHost', 'disable-floc-easily' ); ?></a></li>
                                    <li><a href="<?php echo esc_url( 'https://shareasale.com/r.cfm?b=1320631&u=2323235&m=41388&urllink=&afftrack=0'); ?>"><?php _e( 'Genesis Framework', 'disable-floc-easily' ); ?></a></li>
                                </ul>
                            </div>
                        </div>
                    </sidebar>
                </div>
            </div>
        <?php
        }

        public function dfe_general_settings_method_callback()
        {
            $options = get_option( 'dfe_general_settings' );

            $selectElement = "<select name='dfe_general_settings[dfe_general_settings_select_method]'>";

            $methods = apply_filters( 'dfe_blocking_methods_filter', array() );

            $selectMethod = ! empty( $options['dfe_general_settings_select_method'] ) ? $options['dfe_general_settings_select_method'] : '';

            foreach ($methods as $id => $data) {
                if ( $selectMethod === $id ) {
                    $selected = 'selected="selected"';
                } else {
                    $selected = null;
                }
    
                $selectElement .= "<option value=\"$id\" $selected>${data['title']}</option>";
            }

            $selectElement .= '</select>';
            echo $selectElement;
        }

        public function dfe_delete_options_callback() {

            $options = get_option( 'dfe_general_settings' );
            $ifChecked = ! empty( $options['dfe_delete_options'] ) ? $options['dfe_delete_options'] : '';
        
            $html = '<input type="checkbox" id="dfe_delete_options_cb" name="dfe_general_settings[dfe_delete_options]" value="1"' . checked( 1, $ifChecked, false ) . '/>';
            $html .= '<label for="dfe_delete_options_cb">' . __( "Check the box", "disable-floc-easily") . '</label>';
        
            echo $html;
        
        }

        public function disable_floc_blocking_methods()
        {
            $methods = array(
                'none'      => array(
                    'title'    => __('None', 'disable-floc-easily'),
                ),
                'filter'    => array(
                    'title'     => __('Filter', 'disable-floc-easily'),
                    'callback'  => array( $this, 'add_by_filter_method' ),
                ),
                'apache'    => array(
                    'title'     => __('Apache', 'disable-floc-easily'),
                    'callback'  => array( $this, 'add_by_apache_method' ),
                ),
            );
            return $methods;
        }

        public function add_by_filter_method()
        {
            add_filter( 'wp_headers', array( $this, 'disable_floc_easily_through_wp_filter' ) );
        }

        public function add_by_apache_method()
        {
            add_filter( 'mod_rewrite_rules', array( $this, 'dfe_add_rewrite_rules' ) );
        }

        public function disable_floc_easily_through_wp_filter( $headers )
        {
            // Check if the $headers is an array
            if ( is_array( $headers ) ) {
                // Check if the key 'Permissions-Policy' exists in $headers
                if ( array_key_exists( $this->headerKey, $headers )  ) {
                    // Store the value of $headers['Permissions-Policy'] in variable;
                    $oldValue = $headers[$this->headerKey];
                    // Check if value contains 'interest-cohort=()'
                    if ( stripos ( $oldValue, $this->headerValue ) !== false ) {
                        // Value is in header, return current headers
                        return $headers;
                    } else {
                        // Value isn't in header, so concatenate 'interest-cohort=()' and return $headers
                        $headers[$this->headerKey] .= ', ' .$this->headerValue;
                    }
                } else {
                    // No header with key 'Permissions-Policy' was found, so add it
                    $headers[$this->headerKey] = $this->headerValue;
                }
            }

            return $headers;
        }

        public function dfe_add_rewrite_rules( $rules = '' )
        {
            $rules .= "\n\n# Added by Disable FLoC Easily\n";
            $rules .= "<IfModule mod_headers.c>\n";
            $rules .= "     Header always set Permissions-Policy \"interest-cohort=()\"\n";
            $rules .= "</IfModule>\n";
            
            return $rules;
        }

        public function dfe_flush_rewrite_rules( $value, $oldValue, $option )
        {
            if ( $value['dfe_general_settings_select_method'] === 'apache' || $oldValue['dfe_general_settings_select_method'] === 'apache' ) {
                set_transient('dfe_flush_rules', 'true');
            }
        
            return $value;
        }

        public function dfe_admin_enqueue_scripts()
        {
            wp_enqueue_style( $this->instance . '-css', DFE_PLUGIN_DIR_URI . 'css/admin.css' );
        }

    }
}